import os
import pprint

import aiohttp
import json

import aiosqlite

from loguru import logger

from funciones.mainselenium import verificar_mensajes, monitorear_ventana, enviar_mensaje

from bd.bd import crear_string_insert

logger.add("atwsp.log", enqueue=True)


async def monitorear_mensajes(driver):
    """Esta corutina se encarga de revisar los chats de la izquierda, verificando si entran mensajes nuevos
        y guardandolos en la tabla que corresponda para luego ser enviados a AtApis"""

    rta = verificar_mensajes(driver)

    if rta['Dato']:

        mensajes = rta['Dato']

        async with aiosqlite.connect(os.getenv("RUTA_BD")) as bd:

            str_insert_mensajes = crear_string_insert(*mensajes,
                                                      tabla='mensajes')

            await bd.execute(str_insert_mensajes)

            await bd.commit()

    else:
        return None


async def monitorear_chat(driver):
    """Esta corutina se encarga de monitorear la ventana de un chat que se encuentra abierto, obteniendo asi los
        ultimos mensajes que llegan para luego guardarlos en la tabla correspondiente y posteriormente
        ser enviados a AtApis"""

    rta = monitorear_ventana(driver)

    if rta['Estado'] is False or rta['Dato'] is None:
        return None

    id_ultimo_mensaje = rta['Dato']['id_mensaje']

    async with aiosqlite.connect(os.getenv("RUTA_BD")) as bd:

        bd.row_factory = aiosqlite.Row

        cursor = await bd.execute(f"SELECT * FROM mensajes WHERE id_mensaje='{id_ultimo_mensaje}'")

        ultimo_mensaje = await cursor.fetchone()

        if ultimo_mensaje is not None:
            return None

        rta['Dato']['estado'] = 'pendiente'

        str_insert = crear_string_insert(*rta['Dato'].values(),
                                         tabla='mensajes')

        await bd.execute(str_insert)

        await bd.commit()


async def enviar_pendientes_a_usuarios(driver):
    """Esta corutina se encarga de obtener los mensajes pendientes de enviar a un usuario provenientes de AtApis y que
        se han guardado en la tabla 'fila', luego utiliza el webdriver para enviarlos a los usuarios que correspondan"""

    async with aiosqlite.connect(os.getenv("RUTA_BD")) as bd:
        bd.row_factory = aiosqlite.Row

        async with bd.execute("SELECT rowid,* FROM fila WHERE estado='pendiente'") as cursor:
            async for row in cursor:
                await bd.execute(f"UPDATE fila SET estado='enviando' WHERE rowid={row['rowid']}")

                await bd.commit()

                try:
                    enviar_mensaje(driver,
                                   row['mensaje'],
                                   row['telefono'],
                                   row['nombre_agendado'])
                except Exception as error:
                    logger.info("Falla al enviar el mensaje al usuario {}, Error: {}", row['telefono'], error)
                    return None

                await bd.execute(f"UPDATE fila SET estado='enviado' WHERE rowid={row['rowid']}")

                await bd.commit()


async def obtener_pendientes_y_enviar():
    """Esta corutina se encarga de obtener los mensajes que deben enviarse a AtApis, provenientes de la tabla
        'mensajes', luego se evalua la respuesta de AtApis y se actualiza el estado del mensaje a 'enviado'
        solamente si la respuesta de AtApis llego con estado en True"""

    pendientes = list()  # Inicializamos la lista de pendientes

    telefono_cliente = os.getenv("TELEFONO")

    async with aiosqlite.connect(os.getenv("RUTA_BD")) as bd:
        bd.row_factory = aiosqlite.Row

        async with bd.execute("SELECT * FROM mensajes WHERE estado='pendiente'") as cursor:
            async for row in cursor:
                dic = dict(telefono_origen=row['telefono'].replace(' ', '').replace('-', ''),
                           telefono_destino=telefono_cliente,
                           mensaje=row['mensaje'],
                           nombre=row['nombre_agendado'],
                           hora_mensaje=row['fecha_hora'],
                           clase=row['clase'])

                await bd.execute(f"UPDATE mensajes SET estado='enviando' WHERE id_mensaje='{row['id_mensaje']}'")

                await bd.commit()

                pendientes.append(dic)

            if len(pendientes) == 0:
                return None

            async with aiohttp.ClientSession() as sesion:

                url = os.getenv("IP_ATAPIS") + "/nuevo_mensaje"

                for pendiente in pendientes:

                    async with sesion.post(url=url, json=pendiente) as respuesta:

                        rta_atapis = await respuesta.text()

                        rta_atapis = json.loads(rta_atapis)

                        if rta_atapis['Estado']:

                            str_update = f"UPDATE mensajes SET estado='enviado' WHERE id_mensaje='{pendiente['id_mensaje']}'"

                            await bd.execute(str_update)

                            await bd.commit()
                        else:
                            return None


async def obtener_mensajes_para_enviar():
    async with aiohttp.ClientSession() as sesion:

        url = os.getenv("IP_ATAPIS") + "/msj/obtener_msj_para_enviar"

        datos_request = dict(telefono=os.getenv("TELEFONO"))

        async with sesion.post(url=url, json=datos_request) as respuesta:

            rta_atapis = json.loads(await respuesta.text())

            if rta_atapis['Estado'] is False:
                return rta_atapis

            pendientes = rta_atapis['Dato']

            if len(pendientes) == 0:
                return None

    async with aiosqlite.connect(os.getenv("RUTA_BD")) as bd:

        for pendiente in pendientes:
            str_insert = crear_string_insert(pendiente['telefono_destino'],
                                             pendiente['mensaje'],
                                             'pendiente',
                                             pendiente['telefono_destino'],
                                             pendiente['fecha_hora'],
                                             tabla='fila')

            await bd.execute(str_insert)

            await bd.commit()
