import asyncio
import sys
import signal
import psutil
import os

if sys.platform == 'linux':
    sys.path.append(os.getcwd() + "/funciones")
else:
    sys.path.append(os.getcwd() + "\\funciones")

from dotenv import load_dotenv

from funciones.principales import monitorear_chat, monitorear_mensajes, enviar_pendientes_a_usuarios, \
    obtener_pendientes_y_enviar, obtener_mensajes_para_enviar, logger

from funciones.asincrono import abrir_navegador, apagar

from funciones.utiles import parsear_bytes
from bd.bd import bootstrap_bd

import selenium.common.exceptions as seleniumexc

from datetime import datetime, timedelta

logger.add("hw.log", enqueue=True)

load_dotenv()


async def main():
    # Inicializamos la base de datos, creando las tablas si no existen
    await bootstrap_bd()

    # Conectamos con el driver de selenium, abriendo asi el navegador con whatsapp
    driver = abrir_navegador()

    ultimo_logeo = datetime.now()  # Inicializamos el horario de logeo

    loop = asyncio.get_running_loop()

    # CODIGO DE SEÑALES COMENTADO PORQUE EN WINDOWS NO FUNCIONA
    # # Definimos las señales ante las cuales el programa debe apagarse
    # signals = (signal.SIGHUP, signal.SIGTERM, signal.SIGINT)
    #
    # for s in signals:  # Agregamos las señales al loop
    #     # Nuestro handler para las señales definidas arriba, es la corutina 'apagar', la cual se encargara de
    #     # finalizar la sesion del webdriver (NO LA VENTANA, LA SESION), y de cancelar todas las corutinas pendientes
    #     # liberando asi los recursos que estaban ocupando.
    #     loop.add_signal_handler(
    #         s, lambda s=s: asyncio.create_task(apagar(loop, driver)))

    while True:
        # Con este try verificamos si el webdriver se cerro, en caso de estarlo, enviamos una señal para
        # que se cancelen todas las tareas, liberando asi los recursos que estan ocupando
        try:
            _ = driver.window_handles
        except (seleniumexc.InvalidSessionIdException,
                seleniumexc.WebDriverException):
            logger.warning("Finalizando el programa, se cerro el webdriver")
            # os.kill(os.getpid(), signal.SIGTERM)
            await apagar(loop, driver)
            quit()

        # Se ejecutan las corutinas que interactuan con el driver
        await asyncio.gather(monitorear_chat(driver),
                             monitorear_mensajes(driver),
                             enviar_pendientes_a_usuarios(driver),
                             return_exceptions=True)

        # Se ejecutan las corutinas que no interactuan con el driver
        await asyncio.gather(obtener_mensajes_para_enviar(),
                             obtener_pendientes_y_enviar(),
                             return_exceptions=True)

        # Logearemos el hardware cada 15 minutos
        if datetime.now() - timedelta(minutes=15) >= ultimo_logeo:
            ram = psutil.virtual_memory()

            logger.info("Uso del CPU: {}% | RAM libre: {} | RAM usada: {}",
                        psutil.cpu_percent(), parsear_bytes(ram.available), parsear_bytes(ram.used))

            ultimo_logeo = datetime.now()


if __name__ == "__main__":
    try:
        asyncio.run(main(), debug=False)
    except Exception as e:
        logger.info(e)
